import { createFilter } from 'rollup-pluginutils';
import MagicString from 'magic-string';
import { addon } from 'nano-css/addon/extract';
import { walk } from 'estree-walker';
import { writeFileSync } from 'fs';
import { ensureFileSync } from 'fs-extra';

function nanoCSS(opts) {
	if ( opts === void 0 ) opts = {};


	var filter = createFilter(opts.include, opts.exclude);

	if (!opts.nano) {
		throw new Error('must specify nano');
	}

	var nano = opts.nano;
	addon(nano);

	var originalSheet = nano.sheet;
	var replacements = null;
	nano.sheet = function (map, block) {
		var styles = originalSheet(map, block);
		replacements = styles;
		return styles;
	};

	function finishedGenerate(bundleOpts, bundle, isWrite) {
		if (opts.outputFile && (typeof isWrite === 'undefined' || isWrite)) {
			ensureFileSync(opts.outputFile);
			writeFileSync(opts.outputFile, nano.raw);
		}
	}

	return {
		name: 'nano-css',

		transform: function (code, id) {
			if (!filter(id)) {
				return null;
			}

			var that = this;

			var canRemoveImport = true;

			var magicString = new MagicString(code);
			var result = that.parse(code);

			var importStart = null, importEnd = null, importName = 'nano', foundImport = false;

			walk(result, {
				enter: function enter(node) {
					//Find the import declarations for nano and keep track of it
					if (!node || !node.type) {
						return;
					}

					if (node.type === 'ImportDeclaration' && node.specifiers && !foundImport) {

						var specs = node.specifiers;
						specs.forEach(function (spec) {
							if ((spec.imported && spec.imported.name === 'nano') ||
								(spec.local && spec.local.name === 'nano')
							) {
								importStart = node.start;
								importEnd = node.end;
								importName = spec.local.name;
								foundImport = true;
							}
						});
					}

					if (node.type === 'VariableDeclaration' && node.declarations && node.declarations.length && !foundImport) {
						//Check to see if this declaration is the require('nano'); we're looking for
						var decs = node.declarations;
						decs.forEach(function (dec) {
							if (dec.id) {
								//They might use const nano = require('./nano');
								if (
									dec.id.type === 'Identifier' &&
									dec.id.name === 'nano' &&
									!foundImport
								) {
									foundImport = true;
									importStart = node.start;
									importEnd = node.end;
									importName = dec.id.name;
									foundImport = true;
								}

								//They've might use: const { nano } = require('./nano');
								if (
									dec.id.type === 'ObjectPattern' &&
									dec.id.properties &&
									dec.id.properties.length &&
									!foundImport
								) {
									var props = dec.id.properties;
									props.forEach(function (prop) {
										if (prop.key && prop.key.name === 'nano') {
											foundImport = true;
											importStart = node.start;
											importEnd = node.end;
											importName = prop.key.name;
											foundImport = true;
										}
									});
								}
							}
						});
					}

					//Find any calls to nano.sheet
					if (node.type === 'CallExpression' && node.callee && node.callee.type === 'MemberExpression') {
						var callee = node.callee;
						if (callee.object.name === importName) {
							//If the call was to something *other* than nano.sheet, we can't remove the import
							if (callee.property.name !== 'sheet') {
								canRemoveImport = false;
							}

							if (callee.property.name === 'sheet' && node.arguments) {
								var args = node.arguments;
								if (args.length) {
									replacements = null;
									// console.log('found nano, going to require file', id);
									try {
										var x = require(id);
									} catch (ex) {
										that.warn('Failed to simplify nano call');
										that.warn('Failed to load file: ' + ex.toString());
										// console.error(ex);
										// console.log('------------');
										// console.log(code);
										// console.log('----------');
										// const c = readFileSync(id);
										// console.log(c.toString());
										// console.log('------------');
										// throw ex;
										canRemoveImport = false;
									}
									// console.log('after require');
									if (replacements) {
										magicString.addSourcemapLocation(node.start);
										magicString.addSourcemapLocation(node.end);

										try {
											magicString.overwrite(node.start, node.end, JSON.stringify(replacements));
										} catch (ex) {
											that.warn('Failed to simplify nano call: ' + ex.toString());
											canRemoveImport = false;
										}
									}
								}
							}
						}
					}
				}
			});
			// console.log('before removing import');
			// console.log('----');
			// console.log(magicString.toString());
			// console.log('------');
			if (canRemoveImport && importStart !== null && importEnd !== null) {
				magicString.overwrite(importStart, importEnd, '');
				// console.log('removing import');
			}

			// console.log('going to write code');
			// console.log('------');
			// console.log(magicString.toString());
			// console.log('------');
			return {
				code: magicString.toString(),
				map: magicString.generateMap()
			};

		},
		ongenerate: finishedGenerate,
		generateBundle: finishedGenerate

	};
}

export default nanoCSS;
